[tasque/transition: 97/213] Merged Application.cs into NativeApplication.cs



commit 627c1aea3322708405a0cd5620fab4b24949f0c7
Author: Antonius Riha <antoniusriha gmail com>
Date:   Thu Aug 16 08:54:39 2012 +0200

    Merged Application.cs into NativeApplication.cs
    
    The former all-purpose class Application in Tasque.Gtk/Application.cs
    has been merged into formerly named libtasque/NativeApplication.cs,
    now named libtasque/Application.cs.
    libtasque/Application.cs and Tasque.Gtk/GtkApplication.cs don't serve
    that many static members as their predecessors. Although this results in
    lengthier code than before in many places, it has been done to enable a
    future transition to a more object oriented programming model and to
    support the development of a more usable tasque model library.
    
    * Renamed libtasque/NativeApplication.cs to libtasque/Application.cs.
    * libtasque/libtasque.csproj: Added ref to CollectionView
    * Renamed Tasque.Gtk/GtkApplicationBase.cs to Tasque.Gtk/GtkApplication.cs.
    
    * Added Mono.Options.cs for cmd line parsing and rewrote parsing in
    libtasque/Application.cs using Mono.Options.
    
    * Moved the following files from Tasque.Gtk/ to the model (libtasque):
    	- CompletedTaskGroupModel.cs
    	- Preferences.cs
    	- TaskGroupModel.cs
    
    * GtkWinApplication: Some formatting.
    
    * tasque/tasque.csproj:
    	- Removed unused ref to libtasqueui
    	- Added ref to libtasque
    
    * Program.cs: removed unused using Tasque.UIModel.Legacy
    
    * DummyBackend.csproj: Added refs to gtk and libtasque.
    * DummyPreferences.cs: Removed empty line.

 src/Addins/DummyBackend/DummyBackend.csproj        |   10 +
 src/Addins/DummyBackend/Gtk/DummyPreferences.cs    |    1 -
 src/Options.cs                                     | 1300 ++++++++++++++++++++
 src/Tasque.Gtk/Application.cs                      |  516 --------
 src/Tasque.Gtk/CompletedTaskGroup.cs               |   12 +-
 src/Tasque.Gtk/GtkApplication.cs                   |  224 ++++
 src/Tasque.Gtk/GtkApplicationBase.cs               |   83 --
 src/Tasque.Gtk/GtkLinuxApplication.cs              |    2 +-
 src/Tasque.Gtk/GtkTray.cs                          |   20 +-
 src/Tasque.Gtk/GtkWinApplication.cs                |   14 +-
 src/Tasque.Gtk/PreferencesDialog.cs                |   32 +-
 src/Tasque.Gtk/TaskGroup.cs                        |   11 +-
 src/Tasque.Gtk/TaskTreeView.cs                     |   10 +-
 src/Tasque.Gtk/TaskWindow.cs                       |   50 +-
 src/Tasque.Gtk/Tasque.Gtk.csproj                   |    6 +-
 src/libtasque/Application.cs                       |  388 ++++++
 .../CompletedTaskGroupModel.cs                     |    0
 src/libtasque/INativeApplication.cs                |    2 +-
 src/libtasque/NativeApplication.cs                 |   93 --
 src/{Tasque.Gtk => libtasque}/Preferences.cs       |    0
 src/{Tasque.Gtk => libtasque}/TaskGroupModel.cs    |    0
 src/libtasque/libtasque.csproj                     |   15 +-
 src/tasque/Program.cs                              |    1 -
 src/tasque/tasque.csproj                           |    6 +-
 tasque.sln                                         |    3 +
 25 files changed, 2019 insertions(+), 780 deletions(-)
---
diff --git a/src/Addins/DummyBackend/DummyBackend.csproj b/src/Addins/DummyBackend/DummyBackend.csproj
index a9f2d02..532bb67 100644
--- a/src/Addins/DummyBackend/DummyBackend.csproj
+++ b/src/Addins/DummyBackend/DummyBackend.csproj
@@ -96,5 +96,15 @@
   </ItemGroup>
   <ItemGroup>
     <Reference Include="System" />
+    <Reference Include="gtk-sharp, Version=2.12.0.0, Culture=neutral, PublicKeyToken=35e10195dab3c99f">
+      <Private>False</Private>
+      <Package>gtk-sharp-2.0</Package>
+    </Reference>
+  </ItemGroup>
+  <ItemGroup>
+    <ProjectReference Include="..\..\libtasque\libtasque.csproj">
+      <Project>{784C9AA8-2B28-400B-8CC4-DCDC48CA37F0}</Project>
+      <Name>libtasque</Name>
+    </ProjectReference>
   </ItemGroup>
 </Project>
diff --git a/src/Addins/DummyBackend/Gtk/DummyPreferences.cs b/src/Addins/DummyBackend/Gtk/DummyPreferences.cs
index 1a33a58..f30b73f 100644
--- a/src/Addins/DummyBackend/Gtk/DummyPreferences.cs
+++ b/src/Addins/DummyBackend/Gtk/DummyPreferences.cs
@@ -37,4 +37,3 @@ namespace Tasque.Backends.Dummy.Gtk
 		}
 	}
 }
-
diff --git a/src/Options.cs b/src/Options.cs
new file mode 100644
index 0000000..ecba10b
--- /dev/null
+++ b/src/Options.cs
@@ -0,0 +1,1300 @@
+//
+// Options.cs
+//
+// Authors:
+//  Jonathan Pryor <jpryor novell com>
+//  Federico Di Gregorio <fog initd org>
+//
+// Copyright (C) 2008 Novell (http://www.novell.com)
+// Copyright (C) 2009 Federico Di Gregorio.
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
+// Compile With:
+//   gmcs -debug+ -r:System.Core Options.cs -o:NDesk.Options.dll
+//   gmcs -debug+ -d:LINQ -r:System.Core Options.cs -o:NDesk.Options.dll
+//
+// The LINQ version just changes the implementation of
+// OptionSet.Parse(IEnumerable<string>), and confers no semantic changes.
+
+//
+// A Getopt::Long-inspired option parsing library for C#.
+//
+// NDesk.Options.OptionSet is built upon a key/value table, where the
+// key is a option format string and the value is a delegate that is 
+// invoked when the format string is matched.
+//
+// Option format strings:
+//  Regex-like BNF Grammar: 
+//    name: .+
+//    type: [=:]
+//    sep: ( [^{}]+ | '{' .+ '}' )?
+//    aliases: ( name type sep ) ( '|' name type sep )*
+// 
+// Each '|'-delimited name is an alias for the associated action.  If the
+// format string ends in a '=', it has a required value.  If the format
+// string ends in a ':', it has an optional value.  If neither '=' or ':'
+// is present, no value is supported.  `=' or `:' need only be defined on one
+// alias, but if they are provided on more than one they must be consistent.
+//
+// Each alias portion may also end with a "key/value separator", which is used
+// to split option values if the option accepts > 1 value.  If not specified,
+// it defaults to '=' and ':'.  If specified, it can be any character except
+// '{' and '}' OR the *string* between '{' and '}'.  If no separator should be
+// used (i.e. the separate values should be distinct arguments), then "{}"
+// should be used as the separator.
+//
+// Options are extracted either from the current option by looking for
+// the option name followed by an '=' or ':', or is taken from the
+// following option IFF:
+//  - The current option does not contain a '=' or a ':'
+//  - The current option requires a value (i.e. not a Option type of ':')
+//
+// The `name' used in the option format string does NOT include any leading
+// option indicator, such as '-', '--', or '/'.  All three of these are
+// permitted/required on any named option.
+//
+// Option bundling is permitted so long as:
+//   - '-' is used to start the option group
+//   - all of the bundled options are a single character
+//   - at most one of the bundled options accepts a value, and the value
+//     provided starts from the next character to the end of the string.
+//
+// This allows specifying '-a -b -c' as '-abc', and specifying '-D name=value'
+// as '-Dname=value'.
+//
+// Option processing is disabled by specifying "--".  All options after "--"
+// are returned by OptionSet.Parse() unchanged and unprocessed.
+//
+// Unprocessed options are returned from OptionSet.Parse().
+//
+// Examples:
+//  int verbose = 0;
+//  OptionSet p = new OptionSet ()
+//    .Add ("v", v => ++verbose)
+//    .Add ("name=|value=", v => Console.WriteLine (v));
+//  p.Parse (new string[]{"-v", "--v", "/v", "-name=A", "/name", "B", "extra"});
+//
+// The above would parse the argument string array, and would invoke the
+// lambda expression three times, setting `verbose' to 3 when complete.  
+// It would also print out "A" and "B" to standard output.
+// The returned array would contain the string "extra".
+//
+// C# 3.0 collection initializers are supported and encouraged:
+//  var p = new OptionSet () {
+//    { "h|?|help", v => ShowHelp () },
+//  };
+//
+// System.ComponentModel.TypeConverter is also supported, allowing the use of
+// custom data types in the callback type; TypeConverter.ConvertFromString()
+// is used to convert the value option to an instance of the specified
+// type:
+//
+//  var p = new OptionSet () {
+//    { "foo=", (Foo f) => Console.WriteLine (f.ToString ()) },
+//  };
+//
+// Random other tidbits:
+//  - Boolean options (those w/o '=' or ':' in the option format string)
+//    are explicitly enabled if they are followed with '+', and explicitly
+//    disabled if they are followed with '-':
+//      string a = null;
+//      var p = new OptionSet () {
+//        { "a", s => a = s },
+//      };
+//      p.Parse (new string[]{"-a"});   // sets v != null
+//      p.Parse (new string[]{"-a+"});  // sets v != null
+//      p.Parse (new string[]{"-a-"});  // sets v == null
+//
+
+using System;
+using System.Collections;
+using System.Collections.Generic;
+using System.Collections.ObjectModel;
+using System.ComponentModel;
+using System.Globalization;
+using System.IO;
+using System.Runtime.Serialization;
+using System.Security.Permissions;
+using System.Text;
+using System.Text.RegularExpressions;
+
+#if LINQ
+using System.Linq;
+#endif
+
+#if TEST
+using NDesk.Options;
+#endif
+
+#if NDESK_OPTIONS
+namespace NDesk.Options
+#else
+namespace Mono.Options
+#endif
+{
+	static class StringCoda {
+
+		public static IEnumerable<string> WrappedLines (string self, params int[] widths)
+		{
+			IEnumerable<int> w = widths;
+			return WrappedLines (self, w);
+		}
+
+		public static IEnumerable<string> WrappedLines (string self, IEnumerable<int> widths)
+		{
+			if (widths == null)
+				throw new ArgumentNullException ("widths");
+			return CreateWrappedLinesIterator (self, widths);
+		}
+
+		private static IEnumerable<string> CreateWrappedLinesIterator (string self, IEnumerable<int> widths)
+		{
+			if (string.IsNullOrEmpty (self)) {
+				yield return string.Empty;
+				yield break;
+			}
+			using (IEnumerator<int> ewidths = widths.GetEnumerator ()) {
+				bool? hw = null;
+				int width = GetNextWidth (ewidths, int.MaxValue, ref hw);
+				int start = 0, end;
+				do {
+					end = GetLineEnd (start, width, self);
+					char c = self [end-1];
+					if (char.IsWhiteSpace (c))
+						--end;
+					bool needContinuation = end != self.Length && !IsEolChar (c);
+					string continuation = "";
+					if (needContinuation) {
+						--end;
+						continuation = "-";
+					}
+					string line = self.Substring (start, end - start) + continuation;
+					yield return line;
+					start = end;
+					if (char.IsWhiteSpace (c))
+						++start;
+					width = GetNextWidth (ewidths, width, ref hw);
+				} while (end < self.Length);
+			}
+		}
+
+		private static int GetNextWidth (IEnumerator<int> ewidths, int curWidth, ref bool? eValid)
+		{
+			if (!eValid.HasValue || (eValid.HasValue && eValid.Value)) {
+				curWidth = (eValid = ewidths.MoveNext ()).Value ? ewidths.Current : curWidth;
+				// '.' is any character, - is for a continuation
+				const string minWidth = ".-";
+				if (curWidth < minWidth.Length)
+					throw new ArgumentOutOfRangeException ("widths",
+							string.Format ("Element must be >= {0}, was {1}.", minWidth.Length, curWidth));
+				return curWidth;
+			}
+			// no more elements, use the last element.
+			return curWidth;
+		}
+
+		private static bool IsEolChar (char c)
+		{
+			return !char.IsLetterOrDigit (c);
+		}
+
+		private static int GetLineEnd (int start, int length, string description)
+		{
+			int end = System.Math.Min (start + length, description.Length);
+			int sep = -1;
+			for (int i = start; i < end; ++i) {
+				if (description [i] == '\n')
+					return i+1;
+				if (IsEolChar (description [i]))
+					sep = i+1;
+			}
+			if (sep == -1 || end == description.Length)
+				return end;
+			return sep;
+		}
+	}
+
+	public class OptionValueCollection : IList, IList<string> {
+
+		List<string> values = new List<string> ();
+		OptionContext c;
+
+		internal OptionValueCollection (OptionContext c)
+		{
+			this.c = c;
+		}
+
+		#region ICollection
+		void ICollection.CopyTo (Array array, int index)  {(values as ICollection).CopyTo (array, index);}
+		bool ICollection.IsSynchronized                   {get {return (values as ICollection).IsSynchronized;}}
+		object ICollection.SyncRoot                       {get {return (values as ICollection).SyncRoot;}}
+		#endregion
+
+		#region ICollection<T>
+		public void Add (string item)                       {values.Add (item);}
+		public void Clear ()                                {values.Clear ();}
+		public bool Contains (string item)                  {return values.Contains (item);}
+		public void CopyTo (string[] array, int arrayIndex) {values.CopyTo (array, arrayIndex);}
+		public bool Remove (string item)                    {return values.Remove (item);}
+		public int Count                                    {get {return values.Count;}}
+		public bool IsReadOnly                              {get {return false;}}
+		#endregion
+
+		#region IEnumerable
+		IEnumerator IEnumerable.GetEnumerator () {return values.GetEnumerator ();}
+		#endregion
+
+		#region IEnumerable<T>
+		public IEnumerator<string> GetEnumerator () {return values.GetEnumerator ();}
+		#endregion
+
+		#region IList
+		int IList.Add (object value)                {return (values as IList).Add (value);}
+		bool IList.Contains (object value)          {return (values as IList).Contains (value);}
+		int IList.IndexOf (object value)            {return (values as IList).IndexOf (value);}
+		void IList.Insert (int index, object value) {(values as IList).Insert (index, value);}
+		void IList.Remove (object value)            {(values as IList).Remove (value);}
+		void IList.RemoveAt (int index)             {(values as IList).RemoveAt (index);}
+		bool IList.IsFixedSize                      {get {return false;}}
+		object IList.this [int index]               {get {return this [index];} set {(values as IList)[index] = value;}}
+		#endregion
+
+		#region IList<T>
+		public int IndexOf (string item)            {return values.IndexOf (item);}
+		public void Insert (int index, string item) {values.Insert (index, item);}
+		public void RemoveAt (int index)            {values.RemoveAt (index);}
+
+		private void AssertValid (int index)
+		{
+			if (c.Option == null)
+				throw new InvalidOperationException ("OptionContext.Option is null.");
+			if (index >= c.Option.MaxValueCount)
+				throw new ArgumentOutOfRangeException ("index");
+			if (c.Option.OptionValueType == OptionValueType.Required &&
+					index >= values.Count)
+				throw new OptionException (string.Format (
+							c.OptionSet.MessageLocalizer ("Missing required value for option '{0}'."), c.OptionName), 
+						c.OptionName);
+		}
+
+		public string this [int index] {
+			get {
+				AssertValid (index);
+				return index >= values.Count ? null : values [index];
+			}
+			set {
+				values [index] = value;
+			}
+		}
+		#endregion
+
+		public List<string> ToList ()
+		{
+			return new List<string> (values);
+		}
+
+		public string[] ToArray ()
+		{
+			return values.ToArray ();
+		}
+
+		public override string ToString ()
+		{
+			return string.Join (", ", values.ToArray ());
+		}
+	}
+
+	public class OptionContext {
+		private Option                option;
+		private string                name;
+		private int                   index;
+		private OptionSet             set;
+		private OptionValueCollection c;
+
+		public OptionContext (OptionSet set)
+		{
+			this.set = set;
+			this.c   = new OptionValueCollection (this);
+		}
+
+		public Option Option {
+			get {return option;}
+			set {option = value;}
+		}
+
+		public string OptionName { 
+			get {return name;}
+			set {name = value;}
+		}
+
+		public int OptionIndex {
+			get {return index;}
+			set {index = value;}
+		}
+
+		public OptionSet OptionSet {
+			get {return set;}
+		}
+
+		public OptionValueCollection OptionValues {
+			get {return c;}
+		}
+	}
+
+	public enum OptionValueType {
+		None, 
+		Optional,
+		Required,
+	}
+
+	public abstract class Option {
+		string prototype, description;
+		string[] names;
+		OptionValueType type;
+		int count;
+		string[] separators;
+
+		protected Option (string prototype, string description)
+			: this (prototype, description, 1)
+		{
+		}
+
+		protected Option (string prototype, string description, int maxValueCount)
+		{
+			if (prototype == null)
+				throw new ArgumentNullException ("prototype");
+			if (prototype.Length == 0)
+				throw new ArgumentException ("Cannot be the empty string.", "prototype");
+			if (maxValueCount < 0)
+				throw new ArgumentOutOfRangeException ("maxValueCount");
+
+			this.prototype   = prototype;
+			this.names       = prototype.Split ('|');
+			this.description = description;
+			this.count       = maxValueCount;
+			this.type        = ParsePrototype ();
+
+			if (this.count == 0 && type != OptionValueType.None)
+				throw new ArgumentException (
+						"Cannot provide maxValueCount of 0 for OptionValueType.Required or " +
+							"OptionValueType.Optional.",
+						"maxValueCount");
+			if (this.type == OptionValueType.None && maxValueCount > 1)
+				throw new ArgumentException (
+						string.Format ("Cannot provide maxValueCount of {0} for OptionValueType.None.", maxValueCount),
+						"maxValueCount");
+			if (Array.IndexOf (names, "<>") >= 0 && 
+					((names.Length == 1 && this.type != OptionValueType.None) ||
+					 (names.Length > 1 && this.MaxValueCount > 1)))
+				throw new ArgumentException (
+						"The default option handler '<>' cannot require values.",
+						"prototype");
+		}
+
+		public string           Prototype       {get {return prototype;}}
+		public string           Description     {get {return description;}}
+		public OptionValueType  OptionValueType {get {return type;}}
+		public int              MaxValueCount   {get {return count;}}
+
+		public string[] GetNames ()
+		{
+			return (string[]) names.Clone ();
+		}
+
+		public string[] GetValueSeparators ()
+		{
+			if (separators == null)
+				return new string [0];
+			return (string[]) separators.Clone ();
+		}
+
+		protected static T Parse<T> (string value, OptionContext c)
+		{
+			Type tt = typeof (T);
+			bool nullable = tt.IsValueType && tt.IsGenericType && 
+				!tt.IsGenericTypeDefinition && 
+				tt.GetGenericTypeDefinition () == typeof (Nullable<>);
+			Type targetType = nullable ? tt.GetGenericArguments () [0] : typeof (T);
+			TypeConverter conv = TypeDescriptor.GetConverter (targetType);
+			T t = default (T);
+			try {
+				if (value != null)
+					t = (T) conv.ConvertFromString (value);
+			}
+			catch (Exception e) {
+				throw new OptionException (
+						string.Format (
+							c.OptionSet.MessageLocalizer ("Could not convert string `{0}' to type {1} for option `{2}'."),
+							value, targetType.Name, c.OptionName),
+						c.OptionName, e);
+			}
+			return t;
+		}
+
+		internal string[] Names           {get {return names;}}
+		internal string[] ValueSeparators {get {return separators;}}
+
+		static readonly char[] NameTerminator = new char[]{'=', ':'};
+
+		private OptionValueType ParsePrototype ()
+		{
+			char type = '\0';
+			List<string> seps = new List<string> ();
+			for (int i = 0; i < names.Length; ++i) {
+				string name = names [i];
+				if (name.Length == 0)
+					throw new ArgumentException ("Empty option names are not supported.", "prototype");
+
+				int end = name.IndexOfAny (NameTerminator);
+				if (end == -1)
+					continue;
+				names [i] = name.Substring (0, end);
+				if (type == '\0' || type == name [end])
+					type = name [end];
+				else 
+					throw new ArgumentException (
+							string.Format ("Conflicting option types: '{0}' vs. '{1}'.", type, name [end]),
+							"prototype");
+				AddSeparators (name, end, seps);
+			}
+
+			if (type == '\0')
+				return OptionValueType.None;
+
+			if (count <= 1 && seps.Count != 0)
+				throw new ArgumentException (
+						string.Format ("Cannot provide key/value separators for Options taking {0} value(s).", count),
+						"prototype");
+			if (count > 1) {
+				if (seps.Count == 0)
+					this.separators = new string[]{":", "="};
+				else if (seps.Count == 1 && seps [0].Length == 0)
+					this.separators = null;
+				else
+					this.separators = seps.ToArray ();
+			}
+
+			return type == '=' ? OptionValueType.Required : OptionValueType.Optional;
+		}
+
+		private static void AddSeparators (string name, int end, ICollection<string> seps)
+		{
+			int start = -1;
+			for (int i = end+1; i < name.Length; ++i) {
+				switch (name [i]) {
+					case '{':
+						if (start != -1)
+							throw new ArgumentException (
+									string.Format ("Ill-formed name/value separator found in \"{0}\".", name),
+									"prototype");
+						start = i+1;
+						break;
+					case '}':
+						if (start == -1)
+							throw new ArgumentException (
+									string.Format ("Ill-formed name/value separator found in \"{0}\".", name),
+									"prototype");
+						seps.Add (name.Substring (start, i-start));
+						start = -1;
+						break;
+					default:
+						if (start == -1)
+							seps.Add (name [i].ToString ());
+						break;
+				}
+			}
+			if (start != -1)
+				throw new ArgumentException (
+						string.Format ("Ill-formed name/value separator found in \"{0}\".", name),
+						"prototype");
+		}
+
+		public void Invoke (OptionContext c)
+		{
+			OnParseComplete (c);
+			c.OptionName  = null;
+			c.Option      = null;
+			c.OptionValues.Clear ();
+		}
+
+		protected abstract void OnParseComplete (OptionContext c);
+
+		public override string ToString ()
+		{
+			return Prototype;
+		}
+	}
+
+	public abstract class ArgumentSource {
+
+		protected ArgumentSource ()
+		{
+		}
+
+		public abstract string[] GetNames ();
+		public abstract string Description { get; }
+		public abstract bool GetArguments (string value, out IEnumerable<string> replacement);
+
+		public static IEnumerable<string> GetArgumentsFromFile (string file)
+		{
+			return GetArguments (File.OpenText (file), true);
+		}
+
+		public static IEnumerable<string> GetArguments (TextReader reader)
+		{
+			return GetArguments (reader, false);
+		}
+
+		// Cribbed from mcs/driver.cs:LoadArgs(string)
+		static IEnumerable<string> GetArguments (TextReader reader, bool close)
+		{
+			try {
+				StringBuilder arg = new StringBuilder ();
+
+				string line;
+				while ((line = reader.ReadLine ()) != null) {
+					int t = line.Length;
+
+					for (int i = 0; i < t; i++) {
+						char c = line [i];
+						
+						if (c == '"' || c == '\'') {
+							char end = c;
+							
+							for (i++; i < t; i++){
+								c = line [i];
+
+								if (c == end)
+									break;
+								arg.Append (c);
+							}
+						} else if (c == ' ') {
+							if (arg.Length > 0) {
+								yield return arg.ToString ();
+								arg.Length = 0;
+							}
+						} else
+							arg.Append (c);
+					}
+					if (arg.Length > 0) {
+						yield return arg.ToString ();
+						arg.Length = 0;
+					}
+				}
+			}
+			finally {
+				if (close)
+					reader.Close ();
+			}
+		}
+	}
+
+	public class ResponseFileSource : ArgumentSource {
+
+		public override string[] GetNames ()
+		{
+			return new string[]{"@file"};
+		}
+
+		public override string Description {
+			get {return "Read response file for more options.";}
+		}
+
+		public override bool GetArguments (string value, out IEnumerable<string> replacement)
+		{
+			if (string.IsNullOrEmpty (value) || !value.StartsWith ("@")) {
+				replacement = null;
+				return false;
+			}
+			replacement = ArgumentSource.GetArgumentsFromFile (value.Substring (1));
+			return true;
+		}
+	}
+
+	[Serializable]
+	public class OptionException : Exception {
+		private string option;
+
+		public OptionException ()
+		{
+		}
+
+		public OptionException (string message, string optionName)
+			: base (message)
+		{
+			this.option = optionName;
+		}
+
+		public OptionException (string message, string optionName, Exception innerException)
+			: base (message, innerException)
+		{
+			this.option = optionName;
+		}
+
+		protected OptionException (SerializationInfo info, StreamingContext context)
+			: base (info, context)
+		{
+			this.option = info.GetString ("OptionName");
+		}
+
+		public string OptionName {
+			get {return this.option;}
+		}
+
+		[SecurityPermission (SecurityAction.LinkDemand, SerializationFormatter = true)]
+		public override void GetObjectData (SerializationInfo info, StreamingContext context)
+		{
+			base.GetObjectData (info, context);
+			info.AddValue ("OptionName", option);
+		}
+	}
+
+	public delegate void OptionAction<TKey, TValue> (TKey key, TValue value);
+
+	public class OptionSet : KeyedCollection<string, Option>
+	{
+		public OptionSet ()
+			: this (delegate (string f) {return f;})
+		{
+		}
+
+		public OptionSet (Converter<string, string> localizer)
+		{
+			this.localizer = localizer;
+			this.roSources = new ReadOnlyCollection<ArgumentSource>(sources);
+		}
+
+		Converter<string, string> localizer;
+
+		public Converter<string, string> MessageLocalizer {
+			get {return localizer;}
+		}
+
+		List<ArgumentSource> sources = new List<ArgumentSource> ();
+		ReadOnlyCollection<ArgumentSource> roSources;
+
+		public ReadOnlyCollection<ArgumentSource> ArgumentSources {
+			get {return roSources;}
+		}
+
+
+		protected override string GetKeyForItem (Option item)
+		{
+			if (item == null)
+				throw new ArgumentNullException ("option");
+			if (item.Names != null && item.Names.Length > 0)
+				return item.Names [0];
+			// This should never happen, as it's invalid for Option to be
+			// constructed w/o any names.
+			throw new InvalidOperationException ("Option has no names!");
+		}
+
+		[Obsolete ("Use KeyedCollection.this[string]")]
+		protected Option GetOptionForName (string option)
+		{
+			if (option == null)
+				throw new ArgumentNullException ("option");
+			try {
+				return base [option];
+			}
+			catch (KeyNotFoundException) {
+				return null;
+			}
+		}
+
+		protected override void InsertItem (int index, Option item)
+		{
+			base.InsertItem (index, item);
+			AddImpl (item);
+		}
+
+		protected override void RemoveItem (int index)
+		{
+			Option p = Items [index];
+			base.RemoveItem (index);
+			// KeyedCollection.RemoveItem() handles the 0th item
+			for (int i = 1; i < p.Names.Length; ++i) {
+				Dictionary.Remove (p.Names [i]);
+			}
+		}
+
+		protected override void SetItem (int index, Option item)
+		{
+			base.SetItem (index, item);
+			AddImpl (item);
+		}
+
+		private void AddImpl (Option option)
+		{
+			if (option == null)
+				throw new ArgumentNullException ("option");
+			List<string> added = new List<string> (option.Names.Length);
+			try {
+				// KeyedCollection.InsertItem/SetItem handle the 0th name.
+				for (int i = 1; i < option.Names.Length; ++i) {
+					Dictionary.Add (option.Names [i], option);
+					added.Add (option.Names [i]);
+				}
+			}
+			catch (Exception) {
+				foreach (string name in added)
+					Dictionary.Remove (name);
+				throw;
+			}
+		}
+
+		public new OptionSet Add (Option option)
+		{
+			base.Add (option);
+			return this;
+		}
+
+		sealed class ActionOption : Option {
+			Action<OptionValueCollection> action;
+
+			public ActionOption (string prototype, string description, int count, Action<OptionValueCollection> action)
+				: base (prototype, description, count)
+			{
+				if (action == null)
+					throw new ArgumentNullException ("action");
+				this.action = action;
+			}
+
+			protected override void OnParseComplete (OptionContext c)
+			{
+				action (c.OptionValues);
+			}
+		}
+
+		public OptionSet Add (string prototype, Action<string> action)
+		{
+			return Add (prototype, null, action);
+		}
+
+		public OptionSet Add (string prototype, string description, Action<string> action)
+		{
+			if (action == null)
+				throw new ArgumentNullException ("action");
+			Option p = new ActionOption (prototype, description, 1, 
+					delegate (OptionValueCollection v) { action (v [0]); });
+			base.Add (p);
+			return this;
+		}
+
+		public OptionSet Add (string prototype, OptionAction<string, string> action)
+		{
+			return Add (prototype, null, action);
+		}
+
+		public OptionSet Add (string prototype, string description, OptionAction<string, string> action)
+		{
+			if (action == null)
+				throw new ArgumentNullException ("action");
+			Option p = new ActionOption (prototype, description, 2, 
+					delegate (OptionValueCollection v) {action (v [0], v [1]);});
+			base.Add (p);
+			return this;
+		}
+
+		sealed class ActionOption<T> : Option {
+			Action<T> action;
+
+			public ActionOption (string prototype, string description, Action<T> action)
+				: base (prototype, description, 1)
+			{
+				if (action == null)
+					throw new ArgumentNullException ("action");
+				this.action = action;
+			}
+
+			protected override void OnParseComplete (OptionContext c)
+			{
+				action (Parse<T> (c.OptionValues [0], c));
+			}
+		}
+
+		sealed class ActionOption<TKey, TValue> : Option {
+			OptionAction<TKey, TValue> action;
+
+			public ActionOption (string prototype, string description, OptionAction<TKey, TValue> action)
+				: base (prototype, description, 2)
+			{
+				if (action == null)
+					throw new ArgumentNullException ("action");
+				this.action = action;
+			}
+
+			protected override void OnParseComplete (OptionContext c)
+			{
+				action (
+						Parse<TKey> (c.OptionValues [0], c),
+						Parse<TValue> (c.OptionValues [1], c));
+			}
+		}
+
+		public OptionSet Add<T> (string prototype, Action<T> action)
+		{
+			return Add (prototype, null, action);
+		}
+
+		public OptionSet Add<T> (string prototype, string description, Action<T> action)
+		{
+			return Add (new ActionOption<T> (prototype, description, action));
+		}
+
+		public OptionSet Add<TKey, TValue> (string prototype, OptionAction<TKey, TValue> action)
+		{
+			return Add (prototype, null, action);
+		}
+
+		public OptionSet Add<TKey, TValue> (string prototype, string description, OptionAction<TKey, TValue> action)
+		{
+			return Add (new ActionOption<TKey, TValue> (prototype, description, action));
+		}
+
+		public OptionSet Add (ArgumentSource source)
+		{
+			if (source == null)
+				throw new ArgumentNullException ("source");
+			sources.Add (source);
+			return this;
+		}
+
+		protected virtual OptionContext CreateOptionContext ()
+		{
+			return new OptionContext (this);
+		}
+
+		public List<string> Parse (IEnumerable<string> arguments)
+		{
+			if (arguments == null)
+				throw new ArgumentNullException ("arguments");
+			OptionContext c = CreateOptionContext ();
+			c.OptionIndex = -1;
+			bool process = true;
+			List<string> unprocessed = new List<string> ();
+			Option def = Contains ("<>") ? this ["<>"] : null;
+			ArgumentEnumerator ae = new ArgumentEnumerator (arguments);
+			foreach (string argument in ae) {
+				++c.OptionIndex;
+				if (argument == "--") {
+					process = false;
+					continue;
+				}
+				if (!process) {
+					Unprocessed (unprocessed, def, c, argument);
+					continue;
+				}
+				if (AddSource (ae, argument))
+					continue;
+				if (!Parse (argument, c))
+					Unprocessed (unprocessed, def, c, argument);
+			}
+			if (c.Option != null)
+				c.Option.Invoke (c);
+			return unprocessed;
+		}
+
+		class ArgumentEnumerator : IEnumerable<string> {
+			List<IEnumerator<string>> sources = new List<IEnumerator<string>> ();
+
+			public ArgumentEnumerator (IEnumerable<string> arguments)
+			{
+				sources.Add (arguments.GetEnumerator ());
+			}
+
+			public void Add (IEnumerable<string> arguments)
+			{
+				sources.Add (arguments.GetEnumerator ());
+			}
+
+			public IEnumerator<string> GetEnumerator ()
+			{
+				do {
+					IEnumerator<string> c = sources [sources.Count-1];
+					if (c.MoveNext ())
+						yield return c.Current;
+					else {
+						c.Dispose ();
+						sources.RemoveAt (sources.Count-1);
+					}
+				} while (sources.Count > 0);
+			}
+
+			IEnumerator IEnumerable.GetEnumerator ()
+			{
+				return GetEnumerator ();
+			}
+		}
+
+		bool AddSource (ArgumentEnumerator ae, string argument)
+		{
+			foreach (ArgumentSource source in sources) {
+				IEnumerable<string> replacement;
+				if (!source.GetArguments (argument, out replacement))
+					continue;
+				ae.Add (replacement);
+				return true;
+			}
+			return false;
+		}
+
+		private static bool Unprocessed (ICollection<string> extra, Option def, OptionContext c, string argument)
+		{
+			if (def == null) {
+				extra.Add (argument);
+				return false;
+			}
+			c.OptionValues.Add (argument);
+			c.Option = def;
+			c.Option.Invoke (c);
+			return false;
+		}
+
+		private readonly Regex ValueOption = new Regex (
+			@"^(?<flag>--|-|/)(?<name>[^:=]+)((?<sep>[:=])(?<value>.*))?$");
+
+		protected bool GetOptionParts (string argument, out string flag, out string name, out string sep, out string value)
+		{
+			if (argument == null)
+				throw new ArgumentNullException ("argument");
+
+			flag = name = sep = value = null;
+			Match m = ValueOption.Match (argument);
+			if (!m.Success) {
+				return false;
+			}
+			flag  = m.Groups ["flag"].Value;
+			name  = m.Groups ["name"].Value;
+			if (m.Groups ["sep"].Success && m.Groups ["value"].Success) {
+				sep   = m.Groups ["sep"].Value;
+				value = m.Groups ["value"].Value;
+			}
+			return true;
+		}
+
+		protected virtual bool Parse (string argument, OptionContext c)
+		{
+			if (c.Option != null) {
+				ParseValue (argument, c);
+				return true;
+			}
+
+			string f, n, s, v;
+			if (!GetOptionParts (argument, out f, out n, out s, out v))
+				return false;
+
+			Option p;
+			if (Contains (n)) {
+				p = this [n];
+				c.OptionName = f + n;
+				c.Option     = p;
+				switch (p.OptionValueType) {
+					case OptionValueType.None:
+						c.OptionValues.Add (n);
+						c.Option.Invoke (c);
+						break;
+					case OptionValueType.Optional:
+					case OptionValueType.Required: 
+						ParseValue (v, c);
+						break;
+				}
+				return true;
+			}
+			// no match; is it a bool option?
+			if (ParseBool (argument, n, c))
+				return true;
+			// is it a bundled option?
+			if (ParseBundledValue (f, string.Concat (n + s + v), c))
+				return true;
+
+			return false;
+		}
+
+		private void ParseValue (string option, OptionContext c)
+		{
+			if (option != null)
+				foreach (string o in c.Option.ValueSeparators != null 
+						? option.Split (c.Option.ValueSeparators, c.Option.MaxValueCount - c.OptionValues.Count, StringSplitOptions.None)
+						: new string[]{option}) {
+					c.OptionValues.Add (o);
+				}
+			if (c.OptionValues.Count == c.Option.MaxValueCount || 
+					c.Option.OptionValueType == OptionValueType.Optional)
+				c.Option.Invoke (c);
+			else if (c.OptionValues.Count > c.Option.MaxValueCount) {
+				throw new OptionException (localizer (string.Format (
+								"Error: Found {0} option values when expecting {1}.", 
+								c.OptionValues.Count, c.Option.MaxValueCount)),
+						c.OptionName);
+			}
+		}
+
+		private bool ParseBool (string option, string n, OptionContext c)
+		{
+			Option p;
+			string rn;
+			if (n.Length >= 1 && (n [n.Length-1] == '+' || n [n.Length-1] == '-') &&
+					Contains ((rn = n.Substring (0, n.Length-1)))) {
+				p = this [rn];
+				string v = n [n.Length-1] == '+' ? option : null;
+				c.OptionName  = option;
+				c.Option      = p;
+				c.OptionValues.Add (v);
+				p.Invoke (c);
+				return true;
+			}
+			return false;
+		}
+
+		private bool ParseBundledValue (string f, string n, OptionContext c)
+		{
+			if (f != "-")
+				return false;
+			for (int i = 0; i < n.Length; ++i) {
+				Option p;
+				string opt = f + n [i].ToString ();
+				string rn = n [i].ToString ();
+				if (!Contains (rn)) {
+					if (i == 0)
+						return false;
+					throw new OptionException (string.Format (localizer (
+									"Cannot bundle unregistered option '{0}'."), opt), opt);
+				}
+				p = this [rn];
+				switch (p.OptionValueType) {
+					case OptionValueType.None:
+						Invoke (c, opt, n, p);
+						break;
+					case OptionValueType.Optional:
+					case OptionValueType.Required: {
+						string v     = n.Substring (i+1);
+						c.Option     = p;
+						c.OptionName = opt;
+						ParseValue (v.Length != 0 ? v : null, c);
+						return true;
+					}
+					default:
+						throw new InvalidOperationException ("Unknown OptionValueType: " + p.OptionValueType);
+				}
+			}
+			return true;
+		}
+
+		private static void Invoke (OptionContext c, string name, string value, Option option)
+		{
+			c.OptionName  = name;
+			c.Option      = option;
+			c.OptionValues.Add (value);
+			option.Invoke (c);
+		}
+
+		private const int OptionWidth = 29;
+
+		public void WriteOptionDescriptions (TextWriter o)
+		{
+			foreach (Option p in this) {
+				int written = 0;
+				if (!WriteOptionPrototype (o, p, ref written))
+					continue;
+
+				if (written < OptionWidth)
+					o.Write (new string (' ', OptionWidth - written));
+				else {
+					o.WriteLine ();
+					o.Write (new string (' ', OptionWidth));
+				}
+
+				bool indent = false;
+				string prefix = new string (' ', OptionWidth+2);
+				foreach (string line in GetLines (localizer (GetDescription (p.Description)))) {
+					if (indent) 
+						o.Write (prefix);
+					o.WriteLine (line);
+					indent = true;
+				}
+			}
+
+			foreach (ArgumentSource s in sources) {
+				string[] names = s.GetNames ();
+				if (names == null || names.Length == 0)
+					continue;
+
+				int written = 0;
+
+				Write (o, ref written, "  ");
+				Write (o, ref written, names [0]);
+				for (int i = 1; i < names.Length; ++i) {
+					Write (o, ref written, ", ");
+					Write (o, ref written, names [i]);
+				}
+
+				if (written < OptionWidth)
+					o.Write (new string (' ', OptionWidth - written));
+				else {
+					o.WriteLine ();
+					o.Write (new string (' ', OptionWidth));
+				}
+
+				bool indent = false;
+				string prefix = new string (' ', OptionWidth+2);
+				foreach (string line in GetLines (localizer (GetDescription (s.Description)))) {
+					if (indent) 
+						o.Write (prefix);
+					o.WriteLine (line);
+					indent = true;
+				}
+			}
+		}
+
+		bool WriteOptionPrototype (TextWriter o, Option p, ref int written)
+		{
+			string[] names = p.Names;
+
+			int i = GetNextOptionIndex (names, 0);
+			if (i == names.Length)
+				return false;
+
+			if (names [i].Length == 1) {
+				Write (o, ref written, "  -");
+				Write (o, ref written, names [0]);
+			}
+			else {
+				Write (o, ref written, "      --");
+				Write (o, ref written, names [0]);
+			}
+
+			for ( i = GetNextOptionIndex (names, i+1); 
+					i < names.Length; i = GetNextOptionIndex (names, i+1)) {
+				Write (o, ref written, ", ");
+				Write (o, ref written, names [i].Length == 1 ? "-" : "--");
+				Write (o, ref written, names [i]);
+			}
+
+			if (p.OptionValueType == OptionValueType.Optional ||
+					p.OptionValueType == OptionValueType.Required) {
+				if (p.OptionValueType == OptionValueType.Optional) {
+					Write (o, ref written, localizer ("["));
+				}
+				Write (o, ref written, localizer ("=" + GetArgumentName (0, p.MaxValueCount, p.Description)));
+				string sep = p.ValueSeparators != null && p.ValueSeparators.Length > 0 
+					? p.ValueSeparators [0]
+					: " ";
+				for (int c = 1; c < p.MaxValueCount; ++c) {
+					Write (o, ref written, localizer (sep + GetArgumentName (c, p.MaxValueCount, p.Description)));
+				}
+				if (p.OptionValueType == OptionValueType.Optional) {
+					Write (o, ref written, localizer ("]"));
+				}
+			}
+			return true;
+		}
+
+		static int GetNextOptionIndex (string[] names, int i)
+		{
+			while (i < names.Length && names [i] == "<>") {
+				++i;
+			}
+			return i;
+		}
+
+		static void Write (TextWriter o, ref int n, string s)
+		{
+			n += s.Length;
+			o.Write (s);
+		}
+
+		private static string GetArgumentName (int index, int maxIndex, string description)
+		{
+			if (description == null)
+				return maxIndex == 1 ? "VALUE" : "VALUE" + (index + 1);
+			string[] nameStart;
+			if (maxIndex == 1)
+				nameStart = new string[]{"{0:", "{"};
+			else
+				nameStart = new string[]{"{" + index + ":"};
+			for (int i = 0; i < nameStart.Length; ++i) {
+				int start, j = 0;
+				do {
+					start = description.IndexOf (nameStart [i], j);
+				} while (start >= 0 && j != 0 ? description [j++ - 1] == '{' : false);
+				if (start == -1)
+					continue;
+				int end = description.IndexOf ("}", start);
+				if (end == -1)
+					continue;
+				return description.Substring (start + nameStart [i].Length, end - start - nameStart [i].Length);
+			}
+			return maxIndex == 1 ? "VALUE" : "VALUE" + (index + 1);
+		}
+
+		private static string GetDescription (string description)
+		{
+			if (description == null)
+				return string.Empty;
+			StringBuilder sb = new StringBuilder (description.Length);
+			int start = -1;
+			for (int i = 0; i < description.Length; ++i) {
+				switch (description [i]) {
+					case '{':
+						if (i == start) {
+							sb.Append ('{');
+							start = -1;
+						}
+						else if (start < 0)
+							start = i + 1;
+						break;
+					case '}':
+						if (start < 0) {
+							if ((i+1) == description.Length || description [i+1] != '}')
+								throw new InvalidOperationException ("Invalid option description: " + description);
+							++i;
+							sb.Append ("}");
+						}
+						else {
+							sb.Append (description.Substring (start, i - start));
+							start = -1;
+						}
+						break;
+					case ':':
+						if (start < 0)
+							goto default;
+						start = i + 1;
+						break;
+					default:
+						if (start < 0)
+							sb.Append (description [i]);
+						break;
+				}
+			}
+			return sb.ToString ();
+		}
+
+		private static IEnumerable<string> GetLines (string description)
+		{
+			return StringCoda.WrappedLines (description, 
+					80 - OptionWidth, 
+					80 - OptionWidth - 2);
+		}
+	}
+}
+
diff --git a/src/Tasque.Gtk/CompletedTaskGroup.cs b/src/Tasque.Gtk/CompletedTaskGroup.cs
index a1eee31..4bb5a8a 100644
--- a/src/Tasque.Gtk/CompletedTaskGroup.cs
+++ b/src/Tasque.Gtk/CompletedTaskGroup.cs
@@ -54,7 +54,7 @@ namespace Tasque
 			this.HideWhenEmpty = false;
 			
 			selectedCategory = GetSelectedCategory ();
-			Application.Preferences.SettingChanged += OnSelectedCategorySettingChanged;
+			GtkApplication.Instance.Preferences.SettingChanged += OnSelectedCategorySettingChanged;
 			
 			CreateRangeSlider ();
 			UpdateDateRanges ();
@@ -76,11 +76,11 @@ namespace Tasque
 			
 			// TODO: Set the initial value and range
 			string rangeStr =
-				Application.Preferences.Get (Preferences.CompletedTasksRange);
+				GtkApplication.Instance.Preferences.Get (Preferences.CompletedTasksRange);
 			if (rangeStr == null) {
 				// Set a default value of All
 				rangeStr = ShowCompletedRange.All.ToString ();
-				Application.Preferences.Set (Preferences.CompletedTasksRange,
+				GtkApplication.Instance.Preferences.Set (Preferences.CompletedTasksRange,
 											 rangeStr);
 			}
 			
@@ -115,9 +115,9 @@ namespace Tasque
 		{
 			Category foundCategory = null;
 			
-			string cat = Application.Preferences.Get (Preferences.SelectedCategoryKey);
+			string cat = GtkApplication.Instance.Preferences.Get (Preferences.SelectedCategoryKey);
 			if (cat != null) {
-				var categories = Application.Backend.Categories;
+				var categories = GtkApplication.Instance.Backend.Categories;
 				foundCategory = categories.SingleOrDefault (c => c.Name == cat);
 			}
 			
@@ -134,7 +134,7 @@ namespace Tasque
 				return;
 			
 			this.currentRange = range;
-			Application.Preferences.Set (Preferences.CompletedTasksRange,
+			GtkApplication.Instance.Preferences.Set (Preferences.CompletedTasksRange,
 										 range.ToString ());
 			
 			UpdateDateRanges ();
diff --git a/src/Tasque.Gtk/GtkApplication.cs b/src/Tasque.Gtk/GtkApplication.cs
new file mode 100644
index 0000000..9a1417c
--- /dev/null
+++ b/src/Tasque.Gtk/GtkApplication.cs
@@ -0,0 +1,224 @@
+// Permission is hereby granted, free of charge, to any person obtaining 
+// a copy of this software and associated documentation files (the 
+// "Software"), to deal in the Software without restriction, including 
+// without limitation the rights to use, copy, modify, merge, publish, 
+// distribute, sublicense, and/or sell copies of the Software, and to 
+// permit persons to whom the Software is furnished to do so, subject to 
+// the following conditions: 
+//  
+// The above copyright notice and this permission notice shall be 
+// included in all copies or substantial portions of the Software. 
+//  
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION 
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
+// 
+// Copyright (c) 2008 Novell, Inc. (http://www.novell.com) 
+// Copyright (c) 2012 Antonius Riha
+// 
+// Authors: 
+//      Sandy Armstrong <sanfordarmstrong gmail com>
+//      Antonius Riha <antoniusriha gmail com>
+// 
+using System;
+using System.Diagnostics;
+using Mono.Unix;
+
+#if ENABLE_NOTIFY_SHARP
+using Notifications;
+#endif
+
+namespace Tasque
+{
+	public abstract class GtkApplication : Application
+	{
+		public static GtkApplication Instance {
+			get {
+				if (instance == null)
+					throw new InvalidOperationException ("No instance has been created yet.");
+				return instance;
+			}
+		}
+		
+		static GtkApplication instance;
+		
+		protected GtkApplication ()
+		{
+			instance = this;
+		}
+		
+		protected override void OnInitialize ()
+		{
+//			Catalog.Init ("tasque", GlobalDefines.LocaleDir);
+			Gtk.Application.Init ();
+			GLib.Idle.Add (delegate {
+				InitializeIdle ();
+				return false;
+			});
+			
+			GLib.Timeout.Add (60000, delegate {
+				CheckForDaySwitch ();
+				return true;
+			});
+			
+			base.Initialize ();
+		}
+		
+		protected override void OnInitializeIdle ()
+		{
+			trayIcon = GtkTray.CreateTray ();
+			
+			if (Backend == null) {
+				// Pop open the preferences dialog so the user can choose a
+				// backend service to use.
+				ShowPreferences ();
+			} else if (!QuietStart)
+				TaskWindow.ShowWindow ();
+			
+			if (Backend == null || !Backend.Configured) {
+				GLib.Timeout.Add (1000, new GLib.TimeoutHandler (delegate {
+					RetryBackend ();
+					return Backend == null || !Backend.Configured;
+				}));
+			}
+			
+			base.OnInitializeIdle ();
+		}
+		
+		public override void StartMainLoop ()
+		{
+			Gtk.Application.Run ();
+		}
+		
+		public override void QuitMainLoop ()
+		{
+			Gtk.Application.Quit ();
+		}
+		
+#if ENABLE_NOTIFY_SHARP
+		public void ShowAppNotification (Notification notification)
+		{
+			// TODO: Use this API for newer versions of notify-sharp
+//			notification.AttachToStatusIcon (Tasque.GtkApplication.Instance.Instance.trayIcon);
+			notification.Show ();
+		}
+#endif
+		
+		public void ShowPreferences ()
+		{
+			Trace.TraceInformation ("OnPreferences called");
+			if (preferencesDialog == null) {
+				preferencesDialog = new PreferencesDialog ();
+				preferencesDialog.Hidden += OnPreferencesDialogHidden;
+			}
+			
+			preferencesDialog.Present ();
+		}
+		
+		protected override void OnBackendChanged ()
+		{
+			if (backendWasNullBeforeChange)
+				TaskWindow.Reinitialize (!QuietStart);
+			else
+				TaskWindow.Reinitialize (true);
+			
+			Debug.WriteLine ("Configuration status: {0}", Backend.Configured.ToString ());
+			
+			RebuildTooltipTaskGroupModels ();
+			if (trayIcon != null)
+				trayIcon.RefreshTrayIconTooltip ();
+			
+			base.OnBackendChanged ();
+		}
+		
+		protected override void OnBackendChanging ()
+		{
+			if (Backend != null)
+				UnhookFromTooltipTaskGroupModels ();
+			
+			backendWasNullBeforeChange = Backend == null;
+			
+			base.OnBackendChanging ();
+		}
+		
+		protected override void OnDaySwitched ()
+		{
+			// Reinitialize window according to new date
+			if (TaskWindow.IsOpen)
+				TaskWindow.Reinitialize (true);
+				
+			UnhookFromTooltipTaskGroupModels ();
+			RebuildTooltipTaskGroupModels ();
+			if (trayIcon != null)
+				trayIcon.RefreshTrayIconTooltip ();
+		}
+		
+		protected override void OnQuitting ()
+		{
+			if (Backend != null)
+				UnhookFromTooltipTaskGroupModels ();
+			
+			TaskWindow.SavePosition ();
+			
+			base.OnQuitting ();
+		}
+		
+		protected void OnRemoteInstanceKnocked ()
+		{
+			if (RemoteInstanceKnocked != null)
+				RemoteInstanceKnocked (this, EventArgs.Empty);
+		}
+		
+		protected override event EventHandler RemoteInstanceKnocked;
+		
+		void OnPreferencesDialogHidden (object sender, EventArgs args)
+		{
+			preferencesDialog.Destroy ();
+			preferencesDialog.Hidden -= OnPreferencesDialogHidden;
+			preferencesDialog = null;
+		}
+		
+		void OnTooltipModelChanged (object sender, EventArgs args)
+		{
+			if (trayIcon != null)
+				trayIcon.RefreshTrayIconTooltip ();
+		}
+		
+		void RebuildTooltipTaskGroupModels ()
+		{
+			if (Backend == null || Backend.Tasks == null) {
+				OverdueTasks = null;
+				TodayTasks = null;
+				TomorrowTasks = null;
+				return;
+			}
+
+			OverdueTasks = TaskGroupModelFactory.CreateOverdueModel (Backend.Tasks);
+			TodayTasks = TaskGroupModelFactory.CreateTodayModel (Backend.Tasks);
+			TomorrowTasks = TaskGroupModelFactory.CreateTomorrowModel (Backend.Tasks);
+
+			foreach (var model in new TaskGroupModel [] { OverdueTasks, TodayTasks, TomorrowTasks })
+			{
+				if (model != null)
+					model.CollectionChanged += OnTooltipModelChanged;
+			}
+		}
+		
+		void UnhookFromTooltipTaskGroupModels ()
+		{
+			foreach (var model in new TaskGroupModel[] { OverdueTasks, TodayTasks, TomorrowTasks })
+			{
+				if (model == null)
+					model.CollectionChanged -= OnTooltipModelChanged;
+			}
+		}
+		
+		bool backendWasNullBeforeChange;
+		PreferencesDialog preferencesDialog;
+		GtkTray trayIcon;
+	}
+}
diff --git a/src/Tasque.Gtk/GtkLinuxApplication.cs b/src/Tasque.Gtk/GtkLinuxApplication.cs
index caec039..3d978e6 100644
--- a/src/Tasque.Gtk/GtkLinuxApplication.cs
+++ b/src/Tasque.Gtk/GtkLinuxApplication.cs
@@ -28,7 +28,7 @@ using System.Diagnostics;
 
 namespace Tasque
 {
-	public class GtkLinuxApplication : GtkApplicationBase
+	public class GtkLinuxApplication : GtkApplication
 	{
 		protected override void Dispose (bool disposing)
 		{
diff --git a/src/Tasque.Gtk/GtkTray.cs b/src/Tasque.Gtk/GtkTray.cs
index 83df19d..04120c1 100644
--- a/src/Tasque.Gtk/GtkTray.cs
+++ b/src/Tasque.Gtk/GtkTray.cs
@@ -44,14 +44,14 @@ namespace Tasque
 		protected GtkTray ()
 		{
 			UpdateBackend ();
-			Application.Instance.BackendChanged += HandleBackendChanged;
+			GtkApplication.Instance.BackendChanged += HandleBackendChanged;
 			RefreshTrayIconTooltip ();
 		}
 
 		#region IDisposable implementation
 		public void Dispose ()
 		{
-			var app = Application.Instance;
+			var app = GtkApplication.Instance;
 			if (app != null)
 				app.BackendChanged -= HandleBackendChanged;
 		}
@@ -61,9 +61,9 @@ namespace Tasque
 		{
 			var oldTooltip = Tooltip;
 			
-			var overdueTasks = Application.Instance.OverdueTasks;
-			var todayTasks = Application.Instance.TodayTasks;
-			var tomorrowTasks = Application.Instance.TomorrowTasks;
+			var overdueTasks = GtkApplication.Instance.OverdueTasks;
+			var todayTasks = GtkApplication.Instance.TodayTasks;
+			var tomorrowTasks = GtkApplication.Instance.TomorrowTasks;
 			
 			var sb = new StringBuilder ();
 			if (overdueTasks != null) {
@@ -168,12 +168,12 @@ namespace Tasque
 
 				new ActionEntry ("AboutAction", Stock.About, OnAbout),
 
-				new ActionEntry ("PreferencesAction", Stock.Preferences, delegate { Application.ShowPreferences (); }),
+				new ActionEntry ("PreferencesAction", Stock.Preferences, delegate { GtkApplication.Instance.ShowPreferences (); }),
 
 				new ActionEntry ("RefreshAction", Stock.Execute, Catalog.GetString ("Refresh Tasks ..."),
-				                 null, null, delegate { Application.Backend.Refresh(); }),
+				                 null, null, delegate { GtkApplication.Instance.Backend.Refresh(); }),
 
-				new ActionEntry ("QuitAction", Stock.Quit, delegate { Application.Instance.Quit (); })
+				new ActionEntry ("QuitAction", Stock.Quit, delegate { GtkApplication.Instance.Quit (); })
 			});
 			
 			ToggleTaskWindowAction = new Gtk.Action ("ToggleTaskWindowAction", Catalog.GetString ("Toggle Task Window"));
@@ -187,7 +187,7 @@ namespace Tasque
 
 		void UpdateActionSensitivity ()
 		{
-			var backend = Application.Backend;
+			var backend = GtkApplication.Instance.Backend;
 			bool backendItemsSensitive = (backend != null && backend.Initialized);
 			uiManager.GetAction ("/TrayIconMenu/NewTaskAction").Sensitive = backendItemsSensitive;
 			uiManager.GetAction ("/TrayIconMenu/RefreshAction").Sensitive = backendItemsSensitive;
@@ -197,7 +197,7 @@ namespace Tasque
 		{
 			if (backend != null)
 				backend.BackendInitialized -= HandleBackendInitialized;
-			backend = Application.Backend;
+			backend = GtkApplication.Instance.Backend;
 
 			if (backend != null)
 				backend.BackendInitialized += HandleBackendInitialized;
diff --git a/src/Tasque.Gtk/GtkWinApplication.cs b/src/Tasque.Gtk/GtkWinApplication.cs
index b2d78aa..be71e46 100644
--- a/src/Tasque.Gtk/GtkWinApplication.cs
+++ b/src/Tasque.Gtk/GtkWinApplication.cs
@@ -29,7 +29,7 @@ using System.Threading;
 
 namespace Tasque
 {
-	public class GtkWinApplication : GtkApplicationBase
+	public class GtkWinApplication : GtkApplication
 	{
 		protected override void Dispose (bool disposing)
 		{
@@ -44,15 +44,15 @@ namespace Tasque
 		protected override bool IsRemoteInstanceRunning ()
 		{
 			try {
-				waitHandle = EventWaitHandle.OpenExisting(waitHandleName);
-				waitHandle.Set();
+				waitHandle = EventWaitHandle.OpenExisting (waitHandleName);
+				waitHandle.Set ();
 				return true;
 			} catch (WaitHandleCannotBeOpenedException) {
-				waitHandle = new EventWaitHandle(false, EventResetMode.AutoReset, waitHandleName);
-				Debug.WriteLine("EventWaitHandle created.");
+				waitHandle = new EventWaitHandle (false, EventResetMode.AutoReset, waitHandleName);
+				Debug.WriteLine ("EventWaitHandle created.");
 				
-				var porter = new Thread(new ThreadStart(WaitForAnotherInstance));
-				porter.Start();
+				var porter = new Thread (new ThreadStart (WaitForAnotherInstance));
+				porter.Start ();
 			}
 			return false;
 		}
diff --git a/src/Tasque.Gtk/PreferencesDialog.cs b/src/Tasque.Gtk/PreferencesDialog.cs
index 070f77a..96e0a50 100644
--- a/src/Tasque.Gtk/PreferencesDialog.cs
+++ b/src/Tasque.Gtk/PreferencesDialog.cs
@@ -131,12 +131,12 @@ namespace Tasque
 			backendPage = null;
 			backendPageId = -1;
 			
-			if (Application.Backend != null) {
-				backendPage = (Widget)Application.Backend.Preferences;
+			if (GtkApplication.Instance.Backend != null) {
+				backendPage = (Widget)GtkApplication.Instance.Backend.Preferences;
 				if (backendPage != null) {
 					backendPage.Show ();
 					Label l =
-						new Label (GLib.Markup.EscapeText (Application.Backend.Name));
+						new Label (GLib.Markup.EscapeText (GtkApplication.Instance.Backend.Name));
 					l.UseMarkup = false;
 					l.UseUnderline = false;
 					l.Show ();
@@ -175,7 +175,7 @@ namespace Tasque
 			lblTodaysTaskColor.WidthRequest = 75;
 			lblTodaysTaskColor.Show ();
 
-			Preferences prefs = Application.Preferences;
+			Preferences prefs = GtkApplication.Instance.Preferences;
 			txtTodaysTaskColor = new Entry();
 			txtTodaysTaskColor.Text = prefs.Get (Preferences.TodayTaskTextColor);
 			txtTodaysTaskColor.Changed += OnTxtTodaysTaskColorChanged;
@@ -256,10 +256,10 @@ namespace Tasque
 			// Fill out the ComboBox
 			int i = 0;
 			selectedBackend = -1;
-			foreach (Backend backend in Application.AvailableBackends) {
+			foreach (Backend backend in GtkApplication.Instance.AvailableBackends) {
 				backendComboBox.AppendText (backend.Name);
 				backendComboMap [i] = backend;
-				if (backend == Application.Backend)
+				if (backend == GtkApplication.Instance.Backend)
 					selectedBackend = i;
 				i++;
 			}
@@ -300,7 +300,7 @@ namespace Tasque
 			VBox innerSectionVBox = new VBox (false, 6);
 			hbox = new HBox (false, 6);
 			
-			bool showCompletedTasks = Application.Preferences.GetBool (
+			bool showCompletedTasks = GtkApplication.Instance.Preferences.GetBool (
 											Preferences.ShowCompletedTasksKey);
 			showCompletedTasksCheckButton =
 				new CheckButton (Catalog.GetString ("Sh_ow completed tasks"));
@@ -377,7 +377,7 @@ namespace Tasque
 		{
 			Debug.WriteLine("Loading preferences");
 			categoriesToHide =
-				Application.Preferences.GetStringList (Preferences.HideInAllCategory);
+				GtkApplication.Instance.Preferences.GetStringList (Preferences.HideInAllCategory);
 			//if (categoriesToHide == null || categoriesToHide.Count == 0)
 			//	categoriesToHide = BuildNewCategoryList ();
 		}
@@ -386,7 +386,7 @@ namespace Tasque
 		{
 			// showCompletedTasksCheckbox delegate
 			showCompletedTasksCheckButton.Toggled += delegate {
-				Application.Preferences.SetBool (
+				GtkApplication.Instance.Preferences.SetBool (
 					Preferences.ShowCompletedTasksKey,
 					showCompletedTasksCheckButton.Active);
 			};
@@ -401,7 +401,7 @@ namespace Tasque
 		private void OnTxtTodaysTaskColorChanged (object sender, EventArgs args)
 		{
 			// Save the user preference
-			Application.Preferences.Set (Preferences.TodayTaskTextColor,
+			GtkApplication.Instance.Preferences.Set (Preferences.TodayTaskTextColor,
 			                             ((Entry) sender).Text);
 		}
 
@@ -414,7 +414,7 @@ namespace Tasque
 		private void OnTxtOverdueTaskColorChanged (object sender, EventArgs args)
 		{
 			// Save the user preference
-			Application.Preferences.Set (Preferences.OverdueTaskTextColor,
+			GtkApplication.Instance.Preferences.Set (Preferences.OverdueTaskTextColor,
 			                             ((Entry) sender).Text);
 		}
 
@@ -455,7 +455,7 @@ namespace Tasque
 			}
 			
 			// TODO: Set the new backend
-			Application.Backend = newBackend;
+			GtkApplication.Instance.Backend = newBackend;
 			
 			if (newBackend == null)
 				return;
@@ -480,11 +480,11 @@ namespace Tasque
 			}
 			
 			// Save the user preference
-			Application.Preferences.Set (Preferences.CurrentBackend,
+			GtkApplication.Instance.Preferences.Set (Preferences.CurrentBackend,
 										 newBackend.GetType ().ToString ());
 			
 			//categoriesToHide = BuildNewCategoryList ();
-			//Application.Preferences.SetStringList (Preferences.HideInAllCategory,
+			//GtkApplication.Instance.Preferences.SetStringList (Preferences.HideInAllCategory,
 			//									   categoriesToHide);
 			RebuildCategoryTree ();
 		}
@@ -551,7 +551,7 @@ namespace Tasque
 			else
 				categoriesToHide.Add (category.Name);
 			
-			Application.Preferences.SetStringList (Preferences.HideInAllCategory,
+			GtkApplication.Instance.Preferences.SetStringList (Preferences.HideInAllCategory,
 												   categoriesToHide);
 		}
 		
@@ -566,7 +566,7 @@ namespace Tasque
 		{
 			List<string> list = new List<string> ();
 			TreeModel model;
-			IBackend backend = Application.Backend;
+			IBackend backend = GtkApplication.Instance.Backend;
 			if (backend == null)
 				return list;
 			
diff --git a/src/Tasque.Gtk/TaskTreeView.cs b/src/Tasque.Gtk/TaskTreeView.cs
index fac72b7..f85e528 100644
--- a/src/Tasque.Gtk/TaskTreeView.cs
+++ b/src/Tasque.Gtk/TaskTreeView.cs
@@ -435,7 +435,7 @@ namespace Tasque
 			
 			string formatString = "{0}";
 
-			Preferences prefs = Application.Preferences;
+			Preferences prefs = GtkApplication.Instance.Preferences;
 			string todayTasksColor = prefs.Get (Preferences.TodayTaskTextColor);
 			string overdueTaskColor = prefs.Get (Preferences.OverdueTaskTextColor);
 
@@ -519,7 +519,7 @@ namespace Tasque
 				return;
 			}
 			
-			Preferences prefs = Application.Preferences;
+			Preferences prefs = GtkApplication.Instance.Preferences;
 			int timerSeconds = prefs.GetInt (Preferences.InactivateTimeoutKey);
 			// convert to milliseconds for more granularity
 			long timeout = timerSeconds * 1000;
@@ -623,7 +623,7 @@ namespace Tasque
 			
 			if (task.State == TaskState.Active) {
 				bool showCompletedTasks =
-					Application.Preferences.GetBool (
+					GtkApplication.Instance.Preferences.GetBool (
 						Preferences.ShowCompletedTasksKey);
 				
 				// When showCompletedTasks is true, complete the tasks right
@@ -637,7 +637,7 @@ namespace Tasque
 					
 					// Read the inactivate timeout from a preference
 					int timeout =
-						Application.Preferences.GetInt (Preferences.InactivateTimeoutKey);
+						GtkApplication.Instance.Preferences.GetInt (Preferences.InactivateTimeoutKey);
 					Debug.WriteLine ("Read timeout from prefs: {0}", timeout);
 					InactivateTimer timer =
 						new InactivateTimer (this, iter, task, (uint) timeout);
@@ -688,7 +688,7 @@ namespace Tasque
 			string newText = args.NewText;
 			
 			// Attempt to derive due date information from text.
-			if (Application.Preferences.GetBool (Preferences.ParseDateEnabledKey) &&
+			if (GtkApplication.Instance.Preferences.GetBool (Preferences.ParseDateEnabledKey) &&
 			    task.State == TaskState.Active &&
 			    task.DueDate == DateTime.MinValue) {
 				
diff --git a/src/Tasque.Gtk/TaskWindow.cs b/src/Tasque.Gtk/TaskWindow.cs
index 1938d3e..c8d6178 100644
--- a/src/Tasque.Gtk/TaskWindow.cs
+++ b/src/Tasque.Gtk/TaskWindow.cs
@@ -105,8 +105,8 @@ namespace Tasque
 			// Update the window title
 			Title = string.Format ("Tasque");	
 
-			width = Application.Preferences.GetInt("MainWindowWidth");
-			height = Application.Preferences.GetInt("MainWindowHeight");
+			width = GtkApplication.Instance.Preferences.GetInt("MainWindowWidth");
+			height = GtkApplication.Instance.Preferences.GetInt("MainWindowHeight");
 			
 			if(width == -1)
 				width = 600;
@@ -186,7 +186,7 @@ namespace Tasque
 						Gtk.AccelFlags.Visible);
 			
 			globalKeys.AddAccelerator (delegate (object sender, EventArgs e) {
-				Application.Instance.Quit (); },
+				GtkApplication.Instance.Quit (); },
 						(uint) Gdk.Key.q,
 						Gdk.ModifierType.ControlMask,
 						Gtk.AccelFlags.Visible);
@@ -239,7 +239,7 @@ namespace Tasque
 				OnBackendInitialized();
 			}
 			
-			Application.Preferences.SettingChanged += OnSettingChanged;
+			GtkApplication.Instance.Preferences.SettingChanged += OnSettingChanged;
 		}
 
 		void PopulateWindow()
@@ -359,12 +359,12 @@ namespace Tasque
 			
 			// Set up the combo box (after the above to set the current filter)
 
-			var adapter = new TreeModelListAdapter<Category> (Application.Backend.Categories);
+			var adapter = new TreeModelListAdapter<Category> (GtkApplication.Instance.Backend.Categories);
 			categoryComboBox.Model = adapter;	
 
 			// Read preferences for the last-selected category and select it
 			string selectedCategoryName =
-				Application.Preferences.Get (Preferences.SelectedCategoryKey);
+				GtkApplication.Instance.Preferences.Get (Preferences.SelectedCategoryKey);
 			
 			categoryComboBox.Changed += OnCategoryChanged;
 			
@@ -398,10 +398,10 @@ namespace Tasque
 				lastXPos = x;
 				lastYPos = y;
 				
-				Application.Preferences.SetInt("MainWindowLastXPos", lastXPos);
-				Application.Preferences.SetInt("MainWindowLastYPos", lastYPos);
-				Application.Preferences.SetInt("MainWindowWidth", width);
-				Application.Preferences.SetInt("MainWindowHeight", height);	
+				GtkApplication.Instance.Preferences.SetInt("MainWindowLastXPos", lastXPos);
+				GtkApplication.Instance.Preferences.SetInt("MainWindowLastYPos", lastYPos);
+				GtkApplication.Instance.Preferences.SetInt("MainWindowWidth", width);
+				GtkApplication.Instance.Preferences.SetInt("MainWindowHeight", height);	
 			}
 
 		}
@@ -439,12 +439,12 @@ namespace Tasque
 					}
 					taskWindow.Present();
 				}
-			} else if (Application.Backend != null) {
-				TaskWindow.taskWindow = new TaskWindow(Application.Backend);
+			} else if (GtkApplication.Instance.Backend != null) {
+				TaskWindow.taskWindow = new TaskWindow(GtkApplication.Instance.Backend);
 				if(lastXPos == 0 || lastYPos == 0)
 				{
-					lastXPos = Application.Preferences.GetInt("MainWindowLastXPos");
-					lastYPos = Application.Preferences.GetInt("MainWindowLastYPos");				
+					lastXPos = GtkApplication.Instance.Preferences.GetInt("MainWindowLastXPos");
+					lastYPos = GtkApplication.Instance.Preferences.GetInt("MainWindowLastYPos");				
 				}
 
 				int x = lastXPos;
@@ -831,10 +831,10 @@ namespace Tasque
 			lastXPos = x;
 			lastYPos = y;
 			
-			Application.Preferences.SetInt("MainWindowLastXPos", lastXPos);
-			Application.Preferences.SetInt("MainWindowLastYPos", lastYPos);
-			Application.Preferences.SetInt("MainWindowWidth", width);
-			Application.Preferences.SetInt("MainWindowHeight", height);
+			GtkApplication.Instance.Preferences.SetInt("MainWindowLastXPos", lastXPos);
+			GtkApplication.Instance.Preferences.SetInt("MainWindowLastYPos", lastYPos);
+			GtkApplication.Instance.Preferences.SetInt("MainWindowWidth", width);
+			GtkApplication.Instance.Preferences.SetInt("MainWindowHeight", height);
 
 			Debug.WriteLine("WindowDeleted was called");
 			taskWindow = null;
@@ -922,7 +922,7 @@ namespace Tasque
 			// out of the entered task text.
 			DateTime taskDueDate = DateTime.MinValue;
 			string taskName;
-			if (Application.Preferences.GetBool (Preferences.ParseDateEnabledKey))
+			if (GtkApplication.Instance.Preferences.GetBool (Preferences.ParseDateEnabledKey))
 				TaskParser.Instance.TryParse (
 				                         enteredTaskText,
 				                         out taskName,
@@ -966,7 +966,7 @@ namespace Tasque
 					// the "All" category and if not, select the category
 					// specifically.
 					List<string> categoriesToHide =
-						Application.Preferences.GetStringList (
+						GtkApplication.Instance.Preferences.GetStringList (
 							Preferences.HideInAllCategory);
 					if (categoriesToHide != null && categoriesToHide.Contains (item.Category.Name)) {
 						SelectCategory (item.Category.Name);
@@ -999,7 +999,7 @@ namespace Tasque
 			completedTaskGroup.Refilter (category);
 			
 			// Save the selected category in preferences
-			Application.Preferences.Set (Preferences.SelectedCategoryKey,
+			GtkApplication.Instance.Preferences.Set (Preferences.SelectedCategoryKey,
 										 category.Name);
 		}
 		
@@ -1079,7 +1079,7 @@ namespace Tasque
 					 * here in order to enable changing categories. The list of available categories
 					 * is pre-filtered as to not contain the current category and the AllCategory.
 					 */
-					var cvCategories = new CollectionView<Category> (Application.Backend.Categories);
+					var cvCategories = new CollectionView<Category> (GtkApplication.Instance.Backend.Categories);
 					cvCategories.Filter = c => c != null && !(c is AllCategory) && !c.Equals(clickedTask.Category);
 
 					// The categories submenu is only created in case we actually provide at least one category.
@@ -1121,7 +1121,7 @@ namespace Tasque
 			if (clickedTask == null)
 				return;
 		
-			Application.Backend.DeleteTask(clickedTask);
+			GtkApplication.Instance.Backend.DeleteTask(clickedTask);
 			
 			status = Catalog.GetString ("Task deleted");
 			TaskWindow.ShowStatus (status);
@@ -1171,13 +1171,13 @@ namespace Tasque
 		private void OnBackendSyncFinished ()
 		{
 			Debug.WriteLine("Backend sync finished");
-			if (Application.Backend.Configured) {
+			if (GtkApplication.Instance.Backend.Configured) {
 				string now = DateTime.Now.ToString ();
 				// Translators: This status shows the date and time when the task list was last refreshed
 				status = string.Format (Catalog.GetString ("Tasks loaded: {0}"), now);
 				TaskWindow.lastLoadedTime = now;
 				TaskWindow.ShowStatus (status);
-				RebuildAddTaskMenu ((CollectionView<Category>)Application.Backend.Categories);
+				RebuildAddTaskMenu ((CollectionView<Category>)GtkApplication.Instance.Backend.Categories);
 				addTaskEntry.Sensitive = true;
 				categoryComboBox.Sensitive = true;
 				// Keep insensitive text color
diff --git a/src/Tasque.Gtk/Tasque.Gtk.csproj b/src/Tasque.Gtk/Tasque.Gtk.csproj
index d5859aa..fcf13e6 100644
--- a/src/Tasque.Gtk/Tasque.Gtk.csproj
+++ b/src/Tasque.Gtk/Tasque.Gtk.csproj
@@ -115,25 +115,21 @@
     <Compile Include="..\..\build\GlobalDefines.cs">
       <Link>Properties\GlobalDefines.cs</Link>
     </Compile>
-    <Compile Include="Application.cs" />
     <Compile Include="CellRendererDate.cs" />
     <Compile Include="CompletedTaskGroup.cs" />
-    <Compile Include="CompletedTaskGroupModel.cs" />
     <Compile Include="DateButton.cs" />
     <Compile Include="NoteDialog.cs" />
     <Compile Include="NoteWidget.cs" />
-    <Compile Include="Preferences.cs" />
     <Compile Include="PreferencesDialog.cs" />
     <Compile Include="TaskCalendar.cs" />
     <Compile Include="TaskGroup.cs" />
-    <Compile Include="TaskGroupModel.cs" />
     <Compile Include="TaskGroupModelFactory.cs" />
     <Compile Include="TaskTreeView.cs" />
     <Compile Include="TaskWindow.cs" />
     <Compile Include="Utilities.cs" />
-    <Compile Include="GtkApplicationBase.cs" />
     <Compile Include="StatusIconTray.cs" />
     <Compile Include="GtkTray.cs" />
+    <Compile Include="GtkApplication.cs" />
   </ItemGroup>
   <ItemGroup Condition=" '$(Configuration)' == 'GtkWinDebug' or '$(Configuration)' == 'GtkWinRelease' ">
     <Compile Include="GtkWinApplication.cs" />
diff --git a/src/libtasque/Application.cs b/src/libtasque/Application.cs
new file mode 100644
index 0000000..690f83f
--- /dev/null
+++ b/src/libtasque/Application.cs
@@ -0,0 +1,388 @@
+// 
+// Application.cs
+//  
+// Author:
+//       Antonius Riha <antoniusriha gmail com>
+// 
+// Copyright (c) 2012 Antonius Riha
+// 
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+// 
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
+using System;
+using System.Collections.Generic;
+using System.Diagnostics;
+using System.IO;
+using System.Reflection;
+using Mono.Options;
+
+namespace Tasque
+{
+	public abstract class Application : IDisposable
+	{
+		internal static Application Instance { get; private set; }
+
+		protected Application () : this (Path.Combine (Environment.GetFolderPath (
+				Environment.SpecialFolder.ApplicationData), "tasque")) {}
+		
+		protected Application (string confDir)
+		{
+			if (confDir == null)
+				throw new ArgumentNullException ("confDir");
+			
+			ConfDir = confDir;
+			if (!Directory.Exists (confDir))
+				Directory.CreateDirectory (confDir);
+			
+			CurrentDay = DateTime.Today;
+			
+			Application.Instance = this;
+		}
+		
+		public List<Backend> AvailableBackends {
+			get { return new List<Backend> (availableBackends.Values); }
+//			get { return Application.Instance.availableBackends; }
+		}
+		
+		public TaskGroupModel OverdueTasks { get; protected set; }
+		
+		public TaskGroupModel TodayTasks { get; protected set; }
+		
+		public TaskGroupModel TomorrowTasks { get; protected set; }
+		
+		public Preferences Preferences { get { return preferences; } }
+		
+		public virtual void OpenUrlInBrowser (string url)
+		{
+			try {
+				Process.Start (url);
+			} catch (Exception e) {
+				Trace.TraceError ("Error opening url [{0}]:\n{1}", url, e.ToString ());
+			}
+		}
+		
+		#region Backend
+		public Backend Backend { get { return backend; } set { SetBackend (value); } }
+		
+		public event EventHandler BackendChanged;
+		
+		protected virtual void OnBackendChanged ()
+		{
+			if (BackendChanged != null)
+				BackendChanged (this, EventArgs.Empty);
+		}
+		
+		protected virtual void OnBackendChanging () {}
+		
+		void SetBackend (Backend value)
+		{
+			if (value == backend)
+				return;
+			
+			OnBackendChanging ();
+			
+			if (backend != null) {
+				// Cleanup the old backend
+				try {
+					Debug.WriteLine ("Cleaning up backend: {0}", backend.Name);
+					backend.Cleanup ();
+				} catch (Exception e) {
+					Trace.TraceWarning ("Exception cleaning up '{0}': {1}", backend.Name, e);
+				}
+			}
+				
+			// Initialize the new backend
+			backend = value;
+			if (backend == null)
+				return;
+				
+			Trace.TraceInformation ("Using backend: {0} ({1})", backend.Name, backend.GetType ().ToString ());
+			backend.Initialize ();
+			
+			OnBackendChanged ();
+		}
+		#endregion
+		
+		#region Initialize
+		public void Initialize (string[] args)
+		{
+			if (IsRemoteInstanceRunning ()) {
+				Trace.TraceInformation ("Another instance of Tasque is already running.");
+				Exit (0);
+			}
+			
+			RemoteInstanceKnocked += delegate { ShowMainWindow (); };
+			
+			preferences = new Preferences (ConfDir);
+			
+			ParseArgs (args);
+			
+			SetCustomBackend ();
+			
+			// Discover all available backends
+			LoadAvailableBackends ();
+			
+			OnInitialize ();
+		}
+		
+		protected virtual void OnInitialize () {}
+
+		protected void InitializeIdle ()
+		{
+			if (customBackend != null) {
+				Backend = customBackend;
+			} else {
+				// Check to see if the user has a preference of which backend
+				// to use.  If so, use it, otherwise, pop open the preferences
+				// dialog so they can choose one.
+				var backendTypeString = Preferences.Get (Preferences.CurrentBackend);
+				Debug.WriteLine ("CurrentBackend specified in Preferences: {0}", backendTypeString);
+				if (backendTypeString != null && availableBackends.ContainsKey (backendTypeString))
+					Backend = availableBackends [backendTypeString];
+			}
+			
+			OnInitializeIdle ();
+		}
+		
+		protected virtual void OnInitializeIdle () {}
+		#endregion
+		
+		#region Day switch
+		protected DateTime CurrentDay { get; private set; }
+		
+		protected void CheckForDaySwitch ()
+		{
+			if (DateTime.Today != CurrentDay) {
+				Debug.WriteLine ("Day has changed, reloading tasks");
+				CurrentDay = DateTime.Today;
+				OnDaySwitched ();
+			}
+		}
+		
+		protected virtual void OnDaySwitched () {}
+		#endregion
+		
+		#region Exit and Quit
+		public void Exit (int exitcode)
+		{
+			OnExit (exitcode);
+
+			if (Exiting != null)
+				Exiting (this, EventArgs.Empty);
+
+			Environment.Exit (exitcode);
+		}
+		
+		public event EventHandler Exiting;
+		
+		protected virtual void OnExit (int exitCode) {}
+		
+		public void Quit ()
+		{
+			Trace.TraceInformation ("OnQuit called - terminating application");
+			OnQuitting ();
+			
+			if (backend != null)
+				backend.Cleanup ();
+			
+			QuitMainLoop ();
+		}
+		
+		protected virtual void OnQuitting () {}
+		#endregion
+		
+		#region IDisposable implementation
+		public void Dispose ()
+		{
+			Dispose (true);
+			GC.SuppressFinalize (this);
+		}
+		
+		protected virtual void Dispose (bool disposing) {}
+		
+		~Application ()
+		{
+			Dispose (false);
+		}
+		#endregion
+		
+		#region Main loop hooks
+		public abstract void QuitMainLoop ();
+		
+		public abstract void StartMainLoop ();
+		#endregion
+		
+		#region Single instance
+		//DOCS: Tasque is a single instance app. If Tasque is already started in the current user's
+		// domain, don't start it again. Returns null if no instance found
+		protected abstract bool IsRemoteInstanceRunning ();
+		
+		protected abstract void ShowMainWindow ();
+		
+		protected abstract event EventHandler RemoteInstanceKnocked;
+		#endregion
+		
+		#region Backend loading and helpers
+		protected void RetryBackend()
+		{
+			try {
+				if (backend != null && !backend.Configured) {
+					backend.Cleanup();
+					backend.Initialize();
+				}
+			} catch (Exception e) {
+				Trace.TraceError("{0}", e.Message);
+			}
+		}
+		
+		/// <summary>
+		/// Load all the available backends that Tasque can find.  First look in
+		/// Tasque.exe and then for other DLLs in the same directory Tasque.ex
+		/// resides.
+		/// </summary>
+		void LoadAvailableBackends ()
+		{
+			availableBackends = new Dictionary<string, Backend> ();
+			var backends = new List<Backend> ();
+			var tasqueAssembly = Assembly.GetCallingAssembly ();
+			
+			// Look for other backends in Tasque.exe
+			backends.AddRange (GetBackendsFromAssembly (tasqueAssembly));
+			
+			// Look through the assemblies located in the same directory as Tasque.exe.
+			Debug.WriteLine ("Tasque.exe location: {0}", tasqueAssembly.Location);
+			
+			var loadPathInfo = Directory.GetParent (tasqueAssembly.Location);
+			Trace.TraceInformation ("Searching for Backend DLLs in: {0}", loadPathInfo.FullName);
+			
+			foreach (var fileInfo in loadPathInfo.GetFiles ("*.dll")) {
+				Trace.TraceInformation ("\tReading {0}", fileInfo.FullName);
+				Assembly asm = null;
+				try {
+					asm = Assembly.LoadFile (fileInfo.FullName);
+				} catch (Exception e) {
+					Debug.WriteLine ("Exception loading {0}: {1}", fileInfo.FullName, e.Message);
+					continue;
+				}
+				
+				backends.AddRange (GetBackendsFromAssembly (asm));
+			}
+			
+			foreach (var backend in backends) {
+				string typeId = backend.GetType ().ToString ();
+				if (availableBackends.ContainsKey (typeId))
+					continue;
+				
+				Debug.WriteLine ("Storing '{0}' = '{1}'", typeId, backend.Name);
+				availableBackends [typeId] = backend;
+			}
+		}
+		
+		List<Backend> GetBackendsFromAssembly (Assembly asm)
+		{
+			var backends = new List<Backend> ();
+			Type[] types = null;
+			
+			try {
+				types = asm.GetTypes ();
+			} catch (Exception e) {
+				Trace.TraceWarning ("Exception reading types from assembly '{0}': {1}", asm.ToString (), e.Message);
+				return backends;
+			}
+			
+			foreach (var type in types) {
+				if (!type.IsClass)
+					continue; // Skip non-class types
+				if (type.GetInterface ("Tasque.Backends.IBackend") == null)
+					continue;
+				
+				Debug.WriteLine ("Found Available Backend: {0}", type.ToString ());
+				
+				Backend availableBackend = null;
+				try {
+					availableBackend = (Backend)asm.CreateInstance (type.ToString ());
+				} catch (Exception e) {
+					Trace.TraceWarning ("Could not instantiate {0}: {1}", type.ToString (), e.Message);
+					continue;
+				}
+				
+				if (availableBackend != null)
+					backends.Add (availableBackend);
+			}
+			
+			return backends;
+		}
+		
+		void SetCustomBackend ()
+		{
+			// See if a specific backend is specified
+			if (potentialBackendClassName != null) {
+				Debug.WriteLine ("Backend specified: " + potentialBackendClassName);
+				
+				Assembly asm = Assembly.GetCallingAssembly ();
+				try {
+					customBackend = (Backend)asm.CreateInstance (potentialBackendClassName);
+				} catch (Exception e) {
+					Trace.TraceWarning ("Backend specified on args not found: {0}\n\t{1}",
+						potentialBackendClassName, e.Message);
+				}
+			}
+		}
+		#endregion
+		
+		protected string ConfDir { get; private set; }
+		
+		protected bool QuietStart { get; private set; }
+		
+		void ParseArgs (string[] args)
+		{
+			bool showHelp = false;
+			var p = new OptionSet () {
+				{ "q|quiet", "hide the Tasque window upon start.", v => QuietStart = true },
+				{ "b|backend=", "the name of the {BACKEND} to use.", v => potentialBackendClassName = v },
+				{ "h|help",  "show this message and exit.", v => showHelp = v != null },
+			};
+
+			try {
+				p.Parse (args);
+			} catch (OptionException e) {
+				Console.Write ("tasque: ");
+				Console.WriteLine (e.Message);
+				Console.WriteLine ("Try `tasque --help' for more information.");
+				Exit (-1);
+			}
+
+			if (showHelp) {
+				Console.WriteLine ("Usage: tasque [[-q|--quiet] [[-b|--backend] BACKEND]]");
+				Console.WriteLine ();
+				Console.WriteLine ("Options:");
+				p.WriteOptionDescriptions (Console.Out);
+				Exit (-1);
+			}
+		}
+		
+		/// <value>
+		/// Keep track of the available backends.  The key is the Type name of
+		/// the backend.
+		/// </value>
+		Dictionary<string, Backend> availableBackends;
+		Backend backend;
+		Backend customBackend;
+		string potentialBackendClassName;
+		Preferences preferences;
+	}
+}
diff --git a/src/Tasque.Gtk/CompletedTaskGroupModel.cs b/src/libtasque/CompletedTaskGroupModel.cs
similarity index 100%
rename from src/Tasque.Gtk/CompletedTaskGroupModel.cs
rename to src/libtasque/CompletedTaskGroupModel.cs
diff --git a/src/libtasque/INativeApplication.cs b/src/libtasque/INativeApplication.cs
index 4211330..510331f 100644
--- a/src/libtasque/INativeApplication.cs
+++ b/src/libtasque/INativeApplication.cs
@@ -2,7 +2,7 @@ using System;
 
 namespace Tasque
 {
-	public interface INativeApplication
+	public interface IApplication
 	{
 		string ConfDir { get; }
 
diff --git a/src/Tasque.Gtk/Preferences.cs b/src/libtasque/Preferences.cs
similarity index 100%
rename from src/Tasque.Gtk/Preferences.cs
rename to src/libtasque/Preferences.cs
diff --git a/src/Tasque.Gtk/TaskGroupModel.cs b/src/libtasque/TaskGroupModel.cs
similarity index 100%
rename from src/Tasque.Gtk/TaskGroupModel.cs
rename to src/libtasque/TaskGroupModel.cs
diff --git a/src/libtasque/libtasque.csproj b/src/libtasque/libtasque.csproj
index 660b791..bb1c3a6 100644
--- a/src/libtasque/libtasque.csproj
+++ b/src/libtasque/libtasque.csproj
@@ -105,7 +105,6 @@
     <Compile Include="..\..\build\GlobalDefines.cs">
       <Link>Properties\GlobalDefines.cs</Link>
     </Compile>
-    <Compile Include="NativeApplication.cs" />
     <Compile Include="IBackendPreferences.cs" />
     <Compile Include="Backend.cs" />
     <Compile Include="InternetMode.cs" />
@@ -116,10 +115,18 @@
     <Compile Include="TaskCompletionDateComparer.cs" />
     <Compile Include="SortedNotifyCollection.cs" />
     <Compile Include="ReadOnlySortedNotifyCollection.cs" />
+    <Compile Include="..\Options.cs">
+      <Link>Options.cs</Link>
+    </Compile>
+    <Compile Include="Preferences.cs" />
+    <Compile Include="TaskGroupModel.cs" />
+    <Compile Include="CompletedTaskGroupModel.cs" />
+    <Compile Include="Application.cs" />
   </ItemGroup>
   <ItemGroup>
     <Reference Include="System" />
     <Reference Include="Mono.Posix" />
+    <Reference Include="System.Xml" />
   </ItemGroup>
   <ItemGroup>
     <Folder Include="DateFormatters\" />
@@ -156,4 +163,10 @@
   <Target Name="AfterBuild">
     <Copy SourceFiles="@(DocsSourcesFile)" DestinationFolder="$(OutputPath)" />
   </Target>
+  <ItemGroup>
+    <ProjectReference Include="..\ObservableTransformCollections\CollectionView\CollectionView.csproj">
+      <Project>{A5AAD70F-F4E8-4CAE-A000-01C2D0A10B92}</Project>
+      <Name>CollectionView</Name>
+    </ProjectReference>
+  </ItemGroup>
 </Project>
diff --git a/src/tasque/Program.cs b/src/tasque/Program.cs
index 4362800..f622a37 100644
--- a/src/tasque/Program.cs
+++ b/src/tasque/Program.cs
@@ -28,7 +28,6 @@ using System.Diagnostics;
 using System.Runtime.InteropServices;
 using System.Text;
 using Mono.Unix.Native;
-using Tasque.UIModel.Legacy;
 
 namespace Tasque
 {
diff --git a/src/tasque/tasque.csproj b/src/tasque/tasque.csproj
index 6d4f979..a02ea8d 100644
--- a/src/tasque/tasque.csproj
+++ b/src/tasque/tasque.csproj
@@ -107,9 +107,9 @@
       <Project>{B19B9840-669D-4984-9772-E1F55193A67F}</Project>
       <Name>Tasque.Gtk</Name>
     </ProjectReference>
-    <ProjectReference Include="..\libtasqueui\libtasqueui.csproj">
-      <Project>{DB8F01D5-0161-48E0-B68A-14AEFD22328F}</Project>
-      <Name>libtasqueui</Name>
+    <ProjectReference Include="..\libtasque\libtasque.csproj">
+      <Project>{784C9AA8-2B28-400B-8CC4-DCDC48CA37F0}</Project>
+      <Name>libtasque</Name>
     </ProjectReference>
   </ItemGroup>
 </Project>
diff --git a/tasque.sln b/tasque.sln
index e05dd0f..3ef309f 100644
--- a/tasque.sln
+++ b/tasque.sln
@@ -24,6 +24,9 @@ EndProject
 Project("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}") = "HiveminderBackend", "src\Addins\HiveminderBackend\HiveminderBackend.csproj", "{B3107E6C-08DD-4264-A8D5-467BD96F33CA}"
 EndProject
 Project("{2150E333-8FDC-42A3-9474-1A3956D46DE8}") = "Libraries", "Libraries", "{8F06BE43-859E-4A2D-B620-7CEDFF408F39}"
+	ProjectSection(SolutionItems) = preProject
+		src\Options.cs = src\Options.cs
+	EndProjectSection
 EndProject
 Project("{2150E333-8FDC-42A3-9474-1A3956D46DE8}") = "ObservableCollectionTransforms", "ObservableCollectionTransforms", "{575AA970-031A-4E4A-8409-75D749D3C23E}"
 EndProject



[Date Prev][Date Next]   [Thread Prev][Thread Next]   [Thread Index] [Date Index] [Author Index]
diff --git a/src/Tasque.Gtk/TaskGroup.cs b/src/Tasque.Gtk/TaskGroup.cs
index 138f422..1541b49 100644
--- a/src/Tasque.Gtk/TaskGroup.cs
+++ b/src/Tasque.Gtk/TaskGroup.cs
@@ -63,9 +63,9 @@ namespace Tasque
 			filteredTasks = CreateModel (rangeStart, rangeEnd, tasks);
 
 			filteredTasks.ShowCompletedTasks = 
-				Application.Preferences.GetBool (
+				GtkApplication.Instance.Preferences.GetBool (
 					Preferences.ShowCompletedTasksKey);
-			Application.Preferences.SettingChanged += OnSettingChanged;
+			GtkApplication.Instance.Preferences.SettingChanged += OnSettingChanged;
 			
 			// TODO: Add something to watch events so that the group will
 			// automatically refilter and display/hide itself accordingly.
@@ -371,8 +371,7 @@ namespace Tasque
 		
 		/// <summary>
 		/// This returns the currently selected category.
 		/// TODO: This should really be moved as a method Application or
 		/// or something.
 		/// </summary>
 		/// <returns>
 		/// A <see cref="ICategory"/>
@@ -382,10 +381,10 @@ namespace Tasque
 			// TODO: Move this code into some function in the backend/somewhere
 			// with the signature of GetCategoryForName (string catName):ICategory
 			string selectedCategoryName =
-				Application.Preferences.Get (Preferences.SelectedCategoryKey);
+				GtkApplication.Instance.Preferences.Get (Preferences.SelectedCategoryKey);
 			
 			if (selectedCategoryName != null) {
-				var categories = Application.Backend.Categories;
+				var categories = GtkApplication.Instance.Backend.Categories;
 				return categories.SingleOrDefault (c => c.Name == selectedCategoryName);
 			}
 			
@@ -371,8 +371,7 @@ namespace Tasque
 		/// <summary>
 		/// This returns the currently selected category.
-		/// TODO: This should really be moved as a method Application or
-		/// or something.
+		/// TODO: This should really be moved as a method GtkApplication or something.
 		/// </summary>
 		/// <returns>